import { Observable } from '../core/Observable';
var isEventTarget = function (arg) {
    return typeof arg.addEventListener === 'function' && typeof arg.removeEventListener === 'function';
};
var isEventEmitterA = function (arg) {
    return typeof arg.addListener === 'function' && typeof arg.removeListener === 'function';
};
var fromDOMEvent = function (eventSource, eventName, capture) {
    return new Observable(function (observer) {
        var listener = function (event) { return observer.next(event); };
        try {
            eventSource.addEventListener(eventName, listener, capture);
        }
        catch (error) {
            observer.error(error);
        }
        return function () { return eventSource.removeEventListener(eventName, listener); };
    });
};
var fromEE = function (eventSource, eventName) {
    return new Observable(function (observer) {
        var listener = function () {
            var events = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                events[_i] = arguments[_i];
            }
            events.length > 1 ? observer.next(events) : observer.next(events[0]);
        };
        try {
            eventSource.addListener(eventName, listener);
        }
        catch (error) {
            observer.error(error);
        }
        return function () { return eventSource.removeListener(eventName, listener); };
    });
};
export function fromEvent(eventSource, eventName, capture) {
    if (capture === void 0) { capture = false; }
    if (isEventTarget(eventSource)) {
        return fromDOMEvent(eventSource, eventName, capture);
    }
    if (isEventEmitterA(eventSource)) {
        return fromEE(eventSource, eventName);
    }
    throw new Error('Source must support addEventListener/removeEventListener or addListener/removeListener');
}
