"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Creates a throttled function that only invokes func at most once per every `wait` milliseconds.
 * Function is called on the beginning of `wait` period.
 */
exports.throttle = function (wait, func) {
    var timeoutId;
    return function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        if (timeoutId) {
            return;
        }
        func.apply(this, args);
        timeoutId = setTimeout(function () {
            timeoutId = null;
        }, wait);
    };
};
//# sourceMappingURL=throttle.js.map