"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var debounce_1 = require("../helpers/debounce");
var getSpecies_1 = require("../helpers/getSpecies");
exports.debounceTime = function (dueTime, stream) {
    var C = getSpecies_1.getSpecies(stream);
    return new C(function (observer) {
        var lastValue;
        var lastEmitted = true;
        var debouncedNext = debounce_1.debounce(dueTime, function (value) {
            observer.next(value);
            lastEmitted = true;
        });
        return stream.subscribe({
            next: function (value) {
                lastValue = value;
                lastEmitted = false;
                debouncedNext(value);
            },
            error: function (e) {
                observer.error(e);
            },
            complete: function () {
                if (!lastEmitted) {
                    observer.next(lastValue);
                }
                observer.complete();
            }
        });
    });
};
//# sourceMappingURL=debounceTime.js.map