"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var getSpecies_1 = require("../helpers/getSpecies");
var uniqueId_1 = require("../helpers/uniqueId");
exports.delay = function (wait, stream) {
    if (wait <= 0) {
        return stream;
    }
    var C = getSpecies_1.getSpecies(stream);
    return new C(function (observer) {
        var timers = {};
        var subscription;
        subscription = stream.subscribe({
            start: function (s) {
                subscription = s;
            },
            next: function (value) {
                var id = uniqueId_1.uniqueId();
                timers[id] = setTimeout(function () {
                    observer.next(value);
                    delete timers[id];
                }, wait);
            },
            error: function (reason) {
                observer.error(reason);
            },
            complete: function () {
                timers[uniqueId_1.uniqueId()] = setTimeout(function () {
                    observer.complete();
                }, wait);
            }
        });
        return function () {
            Object.keys(timers).forEach(function (timer) { return clearTimeout(+timer); });
            subscription.unsubscribe();
        };
    });
};
//# sourceMappingURL=delay.js.map