"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var getSpecies_1 = require("../helpers/getSpecies");
var uniqueId_1 = require("../helpers/uniqueId");
exports.mergeAll = function (stream) {
    var C = getSpecies_1.getSpecies(stream);
    return new C(function (observer) {
        var rootCompleted = false;
        var rootSubscription;
        var subscriptions = {};
        rootSubscription = stream.subscribe({
            start: function (s) {
                rootSubscription = s;
            },
            next: function (inner) {
                var id = uniqueId_1.uniqueId();
                try {
                    subscriptions[id] = C.from(inner).subscribe({
                        start: function (s) {
                            subscriptions[id] = s;
                        },
                        next: function (value) {
                            observer.next(value);
                        },
                        error: function (reason) {
                            observer.error(reason);
                        },
                        complete: function () {
                            delete subscriptions[id];
                            if (rootCompleted && Object.keys(subscriptions).length === 0) {
                                observer.complete();
                            }
                        }
                    });
                }
                catch (err) {
                    observer.error(err);
                }
            },
            error: function (reason) {
                observer.error(reason);
            },
            complete: function () {
                rootCompleted = true;
                if (Object.keys(subscriptions).length === 0) {
                    observer.complete();
                }
            }
        });
        return function () {
            rootSubscription.unsubscribe();
            Object.keys(subscriptions).forEach(function (id) {
                subscriptions[id].unsubscribe();
            });
        };
    });
};
//# sourceMappingURL=mergeAll.js.map