"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var getSpecies_1 = require("../helpers/getSpecies");
exports.skipUntil = function (signal, stream) {
    var C = getSpecies_1.getSpecies(stream);
    return new C(function (observer) {
        var blocked = true;
        var signalSubscription;
        var streamSubscription;
        signalSubscription = signal.subscribe({
            start: function (s) {
                signalSubscription = s;
            },
            next: function () {
                blocked = false;
                signalSubscription.unsubscribe();
            },
            error: function (reason) {
                observer.error(reason);
            },
            complete: function () {
                /* istanbul ignore next */
                if (blocked) {
                    observer.complete();
                }
            }
        });
        if (observer.closed) {
            return undefined;
        }
        streamSubscription = stream.subscribe({
            start: function (s) {
                streamSubscription = s;
            },
            next: function (value) {
                if (!blocked) {
                    observer.next(value);
                }
            },
            error: function (reason) {
                observer.error(reason);
            },
            complete: function () {
                observer.complete();
            }
        });
        return function () {
            signalSubscription.unsubscribe();
            streamSubscription.unsubscribe();
        };
    });
};
//# sourceMappingURL=skipUntil.js.map